@@ -50,8 +50,10 @@ bootdata.trans.msm <- function(x) {
     data.boot <- as.data.frame(data.boot)
     ## label every transition in new data as from a different subject
     data.boot[,un["subject"]] <- rep(1:ntrans, each=2)
-
-    for (i in names(dat)[sapply(dat, is.factor)])
+    ## make sure factor covariates retain their original level labels
+    facvars <- names(dat)[sapply(dat, is.factor)] 
+    faccovs <- setdiff(facvars, un)
+    for (i in faccovs)
         data.boot[,i] <- factor(data.boot[,i], labels=sort(unique(dat[,i])))
     data.boot
 }
