@@ -29,6 +29,7 @@ hmmfn HMODELS[] = {
     hmmWeibull,
     hmmPois,
     hmmBinom,
+    hmmBetaBinom,
     hmmTNorm,
     hmmMETNorm,
     hmmMEUnif,
@@ -49,6 +50,7 @@ dhmmfn DHMODELS[] = {
     DhmmWeibull,
     DhmmPois,
     DhmmBinom,
+    DhmmBetaBinom,
     DhmmTNorm,
     DhmmMETNorm,
     DhmmMEUnif,
@@ -129,8 +131,6 @@ void GetCensored (double obs, cmodel *cm, int *nc, double **states)
   pout =   if i in curr 1, else 0
 */
 
-/* TODO does find_exactdeath_hmm need updating for multivariate observations with different models ? */
-
 /* New obstrue facility 
 On entry, obstrue will contain 0 if state unknown, and state if state known
 But how do we know if there are any extra outcome data in the outcome variable?
@@ -142,7 +142,6 @@ If this is a general outcome,  get prob of observing it from HMODELS
 void GetOutcomeProb(double *pout, double *outcome, int nc, int nout, double *hpars, hmodel *hm, qmodel *qm, int obstrue)
 {
     int i, j, k, ind;
-
     for (i=0; i<qm->nst; ++i) {
 	if (hm->hidden && (obstrue==0)) { /* HMMs with true state not known */
 	    if (nout > 1) {  /* multivariate outcomes. Censored states not supported */ 
@@ -157,7 +156,7 @@ void GetOutcomeProb(double *pout, double *outcome, int nc, int nout, double *hpa
 		}
 	    } else {  /* Standard univariate HMM (with or without censored state) */
 		pout[i] = 0;
-		for (j=0; j<nc; ++j){		    
+		for (j=0; j<nc; ++j){
 		    pout[i] += (HMODELS[hm->models[i]])(outcome[j], &(hpars[hm->firstpar[i]]));
 		}
 	    }
@@ -276,11 +275,13 @@ void GetDOutcomeProb(double *dpout, /* qm->nst x hm->nopt */
 	}
 	else {
 	    for (l=0; l<hm->nopt; ++l)
-		dpout[MI(i,l,qm->nst)] = 0;	    
-	    if (nout > 1 && hm->mv)
+		dpout[MI(i,l,qm->nst)] = 0;
+	    if (hm->hidden){
+	      if (nout > 1 && hm->mv)
 		for (r=0; r<nout; ++r)
-		    p += hm->npars[MI(r,i,nout)];
-	    else p += hm->npars[i];
+		  p += hm->npars[MI(r,i,nout)];
+	      else { p += hm->npars[i]; }
+	    }
 	}
     }
     Free(dptmp);
@@ -353,15 +354,20 @@ void calc_dp(msmdata *d, qmodel *qm, double *dpmat)
  * hmmIdent(obs).  This function also works for non-HMM censoring
  * models, just returning the observed state. */
 
+/* For multivariate HMMs, the hmmIdent observation must be in the
+   first outcome */
+
 int find_exactdeath_hmm(double *outcome, int obsno, msmdata *d, qmodel *qm, hmodel *hm){
-    int ideath;
+  int ideath, ind;
     double *hpars = &(hm->pars[MI(0, obsno, hm->totpars)]);
     if (!hm->hidden || d->obstrue[obsno])
 	ideath = outcome[0] - 1;
     else
-	for (ideath=0; ideath < qm->nst; ++ideath)
-	    if (hm->models[ideath] == 1 && hmmIdent(outcome[0], &(hpars[hm->firstpar[ideath]])))
+      for (ideath=0; ideath < qm->nst; ++ideath){
+	ind = (hm->mv ? MI(0,ideath,d->nout) : ideath);
+	    if (hm->models[ind] == 1 && hmmIdent(outcome[0], &(hpars[hm->firstpar[ind]])))
 		break;
+      }	    
     return ideath;
 }
 
@@ -384,11 +390,12 @@ void update_likhidden(double *outcome, int nc, int obsno, msmdata *d, qmodel *qm
 	    newp[j] = 0.0;
 	    for(i = 0; i < qm->nst; ++i)
 		{
-		    if (d->obstype[obsno] == OBS_DEATH)
+		  if (d->obstype[obsno] == OBS_DEATH){
 			T = pmat[MI(i,j,qm->nst)] * qmat[MI(j,ideath,qm->nst)];
+		  }
 		    else {
 			T = pmat[MI(i, j, qm->nst)] * pout[j];
-//			printf("pmat[%d,%d]=%16.12lf,pout[%d]=%16.12lf\n,", i, j, pmat[MI(i, j, qm->nst)], j, pout[j]);
+			//			printf("pmat[%d,%d]=%16.12lf,pout[%d]=%16.12lf\n,", i, j, pmat[MI(i, j, qm->nst)], j, pout[j]);
 
 		    }
 		    if (T < 0) T = 0;
